@@ -16,6 +16,10 @@ import re
 from .sysinfo import RUNNING_ON_APPVEYOR as APPVEYOR
 from .sysinfo import RUNNING_ON_TRAVIS as TRAVIS
 from .sysinfo import RESOLVER_NOT_SYSTEM as ARES
+from .sysinfo import RESOLVER_ARES
+from .sysinfo import RESOLVER_DNSPYTHON
+from .sysinfo import RUNNING_ON_CI
+from .sysinfo import RUNNING_ON_MUSLLINUX
 from .sysinfo import RUN_COVERAGE
 
 
@@ -23,10 +27,13 @@ from .sysinfo import PYPY
 from .sysinfo import PYPY3
 from .sysinfo import PY3
 from .sysinfo import PY2
-from .sysinfo import PY34
 from .sysinfo import PY35
 from .sysinfo import PY36
 from .sysinfo import PY37
+from .sysinfo import PY38
+from .sysinfo import PY39
+from .sysinfo import PY310
+from .sysinfo import PY311
 
 from .sysinfo import WIN
 from .sysinfo import OSX
@@ -112,8 +119,16 @@ def get_switch_expected(fullname):
 
 
 disabled_tests = [
+    # XXX: While we debug latest updates. This is leaking
+    'test_threading.ThreadTests.test_no_refcycle_through_target',
+
     # The server side takes awhile to shut down
     'test_httplib.HTTPSTest.test_local_bad_hostname',
+    # These were previously 3.5+ issues (same as above)
+    # but have been backported.
+    'test_httplib.HTTPSTest.test_local_good_hostname',
+    'test_httplib.HTTPSTest.test_local_unknown_cert',
+
 
     'test_threading.ThreadTests.test_PyThreadState_SetAsyncExc',
     # uses some internal C API of threads not available when threads are emulated with greenlets
@@ -159,18 +174,25 @@ disabled_tests = [
     # the file to a temp location during patching.
     'test_asyncore.HelperFunctionTests.test_compact_traceback',
 
-    'test_signal.WakeupSignalTests.test_wakeup_fd_early',
     # expects time.sleep() to return prematurely in case of a signal;
-    # gevent.sleep() is better than that and does not get interrupted (unless signal handler raises an error)
+    # gevent.sleep() is better than that and does not get interrupted
+    # (unless signal handler raises an error)
+    'test_signal.WakeupSignalTests.test_wakeup_fd_early',
 
+    # expects select.select() to raise select.error(EINTR'interrupted
+    # system call') gevent.select.select() does not get interrupted
+    # (unless signal handler raises an error) maybe it should?
     'test_signal.WakeupSignalTests.test_wakeup_fd_during',
-    # expects select.select() to raise select.error(EINTR'interrupted system call')
-    # gevent.select.select() does not get interrupted (unless signal handler raises an error)
-    # maybe it should?
 
+    # these rely on os.read raising EINTR which never happens with gevent.os.read
     'test_signal.SiginterruptTest.test_without_siginterrupt',
     'test_signal.SiginterruptTest.test_siginterrupt_on',
-    # these rely on os.read raising EINTR which never happens with gevent.os.read
+    'test_signal.SiginterruptTest.test_siginterrupt_off',
+    # This one takes forever and relies on threading details
+    'test_signal.StressTest.test_stress_modifying_handlers',
+    # This uses an external file, and launches it. This means that it's not
+    # actually testing gevent because there's no monkey-patch.
+    'test_signal.PosixTests.test_interprocess_signal',
 
     'test_subprocess.ProcessTestCase.test_leak_fast_process_del_killed',
     'test_subprocess.ProcessTestCase.test_zombie_fast_process_del',
@@ -184,6 +206,10 @@ disabled_tests = [
     # since November of 2016 on Python 3, but now also seen on Python 2/Pypy.
     'test_subprocess.ProcessTestCase.test_leaking_fds_on_error',
 
+    # Added between 3.6.0 and 3.6.3, uses _testcapi and internals
+    # of the subprocess module. Backported to Python 2.7.16.
+    'test_subprocess.POSIXProcessTestCase.test_stopped',
+
     'test_ssl.ThreadedTests.test_default_ciphers',
     'test_ssl.ThreadedTests.test_empty_cert',
     'test_ssl.ThreadedTests.test_malformed_cert',
@@ -213,14 +239,107 @@ disabled_tests = [
     # Does not exist in the test suite until 2.7.4+. Subclasses Popen, and overrides
     # _execute_child. But our version has a different parameter list than the
     # version that comes with PyPy/CPython, so fails with a TypeError.
+
+    # This one crashes the interpreter if it has a bug parsing the
+    # invalid data.
+    'test_ssl.BasicSocketTests.test_parse_cert_CVE_2019_5010',
+    # We had to copy in a newer version of the test file for SSL fixes
+    # and this doesn't work reliably on all versions.
+    'test_httplib.HeaderTests.test_headers_debuglevel',
+
+    # On Appveyor with Python 3.8.0 and 3.7.5, this test
+    # for __class_getitem__ fails. Presumably this was added
+    # in a patch release (it's not in the PEP.) Sigh.
+    # https://bugs.python.org/issue38979
+    'test_context.ContextTest.test_contextvar_getitem',
+    # The same patch that fixed that removed this test,
+    # because it would now fail.
+    'test_context.ContextTest.test_context_var_new_2',
 ]
 
+
+if sys.version_info[:3] < (2, 7, 18):
+    # The final release was 2.7.18. It added some new tests for new
+    # fixes. At this writing, AppVeyor is still on 2.7.17.
+    disabled_tests += [
+        'test_urllib2.MiscTests.test_url_host_with_control_char_rejected',
+    ]
+
+if OSX:
+    disabled_tests += [
+        # These are timing dependent, and sometimes run into the OS X
+        # kernel bug leading to 'Protocol wrong type for socket'.
+        # See discussion at https://github.com/benoitc/gunicorn/issues/1487
+        'test_ssl.SimpleBackgroundTests.test_connect_capath',
+        'test_ssl.SimpleBackgroundTests.test_connect_with_context',
+    ]
+    if PYPY and PY2:
+        disabled_tests += [
+            # This is broken in a standard download of PyPy.
+            'test_subprocess.ProcessTestCase.test_executable_with_cwd',
+        ]
+
+if PYPY and PY2 and WIN:
+    disabled_tests += [
+        # XXX: New in PyPy 7.3.7. This times out. This is testing for
+        # whether \0 in environment keys or values are excluded; they
+        # are, before any waiting is done. The same goes for an '=' in
+        # the key. It looks like we "hang" on the last clause that
+        # tests when there is an '=' in the *value*. It's utterly
+        # unclear to me why that causes an issue; we use the same
+        # underlying CreateProcess call that PyPy itself does, and
+        # no-where before that is the environment manipulated
+        'test_subprocess.ProcesstestCase.test_invalid_env',
+    ]
+
+if PYPY and PY37:
+    disabled_tests += [
+        # The exact error message the code code checks for is different
+        # (possibly just on macOS?). Plain PyPy3 fails as well.
+        'test_signal.WakeupSignalTests.test_wakeup_write_error',
+    ]
+
 if 'thread' in os.getenv('GEVENT_FILE', ''):
     disabled_tests += [
         'test_subprocess.ProcessTestCase.test_double_close_on_error'
         # Fails with "OSError: 9 invalid file descriptor"; expect GC/lifetime issues
     ]
 
+if PY2 and PYPY:
+    disabled_tests += [
+        # These appear to hang or take a long time for some reason?
+        # Likely a hostname/binding issue or failure to properly close/gc sockets.
+        'test_httpservers.BaseHTTPServerTestCase.test_head_via_send_error',
+        'test_httpservers.BaseHTTPServerTestCase.test_head_keep_alive',
+        'test_httpservers.BaseHTTPServerTestCase.test_send_blank',
+        'test_httpservers.BaseHTTPServerTestCase.test_send_error',
+        'test_httpservers.BaseHTTPServerTestCase.test_command',
+        'test_httpservers.BaseHTTPServerTestCase.test_handler',
+        'test_httpservers.CGIHTTPServerTestcase.test_post',
+        'test_httpservers.CGIHTTPServerTestCase.test_query_with_continuous_slashes',
+        'test_httpservers.CGIHTTPServerTestCase.test_query_with_multiple_question_mark',
+        'test_httpservers.CGIHTTPServerTestCase.test_os_environ_is_not_altered',
+
+        # This one sometimes results on connection refused
+        'test_urllib2_localnet.TestUrlopen.test_info',
+        # Sometimes hangs
+        'test_ssl.ThreadedTests.test_socketserver',
+        # We had to update 'CERTFILE' to continue working, but
+        # this test hasn't been updated yet (the CPython tests
+        # are also too new to run on PyPy).
+        'test_ssl.BasicSocketTests.test_parse_cert',
+
+    ]
+
+if PY2 and WIN:
+    disabled_tests += [
+        # This test randomly produces a 'LoopExit: Would block forever'
+        # on 'self.serv.accept()', but only on Windows with Python 2. Possibly
+        # due to the weird refcounting involving socket.makefile (just a guess)?
+        # Seen in both PyPy 7.3 and CPython 2.7.x
+        # https://ci.appveyor.com/project/denik/gevent/builds/36874106/job/guyq6h9k56n81uf6#L563
+        'test_socket.BasicTCPTest2.testDup',
+    ]
 
 if LIBUV:
     # epoll appears to work with these just fine in some cases;
@@ -410,6 +529,9 @@ if LIBUV:
             'test_smtplib.TooLongLineTests.testLineTooLong',
             'test_smtplib.GeneralTests.testTimeoutValue',
 
+            # This sometimes crashes, which can't be our fault?
+            'test_ssl.BasicSocketTests.test_parse_cert_CVE_2019_5010',
+
         ]
 
         if PYPY:
@@ -462,13 +584,17 @@ if LIBUV:
             'test_ssl.ThreadedTests.test_handshake_timeout',
 
             # These sometimes raise LoopExit, for no apparent reason,
-            # mostly but not exclusively on Python 2.
+            # mostly but not exclusively on Python 2. Sometimes (often?)
+            # this happens in the setUp() method when we attempt to get a client
+            # connection
             'test_socket.BufferIOTest.testRecvFromIntoBytearray',
             'test_socket.BufferIOTest.testRecvFromIntoArray',
             'test_socket.BufferIOTest.testRecvIntoArray',
+            'test_socket.BufferIOTest.testRecvIntoMemoryview',
             'test_socket.BufferIOTest.testRecvFromIntoEmptyBuffer',
             'test_socket.BufferIOTest.testRecvFromIntoMemoryview',
             'test_socket.BufferIOTest.testRecvFromIntoSmallBuffer',
+            'test_socket.BufferIOTest.testRecvIntoBytearray',
         ]
 
         if PY3:
@@ -521,6 +647,77 @@ if PY2:
             'test_ssl.ThreadedTests.test_alpn_protocols',
         ]
 
+    disabled_tests += [
+        # At least on OSX, this results in connection refused
+        'test_urllib2_localnet.TestUrlopen.test_https_sni',
+    ]
+
+    if sys.version_info[:3] < (2, 7, 16):
+        # We have 2.7.16 tests; older versions can fail
+        # to validate some SSL things or are missing important support functions
+        disabled_tests += [
+            # Support functions
+            'test_thread.ThreadRunningTests.test_nt_and_posix_stack_size',
+            'test_thread.ThreadRunningTests.test_save_exception_state_on_error',
+            'test_thread.ThreadRunningTests.test_starting_threads',
+            'test_thread.BarrierTest.test_barrier',
+            # Broken SSL
+            'test_urllib2_localnet.TestUrlopen.test_https',
+            'test_ssl.ContextTests.test__create_stdlib_context',
+            'test_ssl.ContextTests.test_create_default_context',
+            'test_ssl.ContextTests.test_options',
+        ]
+
+if PYPY and sys.pypy_version_info[:2] == (7, 3): # pylint:disable=no-member
+
+    if OSX:
+        disabled_tests += [
+            # This is expected to produce an SSLError, but instead it appears to
+            # actually work. See above for when it started failing the same on
+            # Travis.
+            'test_ssl.ThreadedTests.test_alpn_protocols',
+            # This fails, presumably due to the OpenSSL it's compiled with.
+            'test_ssl.ThreadedTests.test_default_ecdh_curve',
+        ]
+
+if PYPY3 and TRAVIS:
+    disabled_tests += [
+        # If socket.SOCK_CLOEXEC is defined, this creates a socket
+        # and tests its type with ``sock.type & socket.SOCK_CLOEXEC``
+        # We have a ``@property`` for ``type`` that takes care of
+        # ``SOCK_NONBLOCK`` on Linux, but otherwise it's just a pass-through.
+        # This started failing with PyPy 7.3.1 and it's not clear why.
+        'test_socket.InheritanceTest.test_SOCK_CLOEXEC',
+    ]
+
+if PYPY3 and WIN:
+    disabled_tests += [
+        # test_httpservers.CGIHTTPServerTestCase all seem to hang.
+        # There seem to be some general subprocess issues. This is
+        # ignored entirely from known_failures.py
+
+        # This produces:
+        #
+        #  OSError: [Errno 10014] The system detected an invalid
+        #  pointer address in attempting to use a pointer argument in
+        #  a call
+        #
+        # When calling socket.socket(fileno=fd) when we actually
+        # call ``self._socket =self._gevent_sock_class()``.
+        'test_socket.GeneralModuleTests.test_socket_fileno',
+
+        # This doesn't respect the scope properly
+        #
+        #  self.assertEqual(sockaddr, ('ff02::1de:c0:face:8d', 1234, 0, ifindex))
+        #   AssertionError: Tuples differ: ('ff02::1de:c0:face:8d%42', 1234, 0, 42) != ('ff02::1de:c0:face:8d', 1234, 0, 42
+        #
+        'test_socket.GeneralModuleTests.test_getaddrinfo_ipv6_scopeid_numeric',
+
+        # self.assertEqual(newsock.get_inheritable(), False)
+        #  AssertionError: True != False
+        'test_socket.InheritanceTest.test_dup',
+    ]
+
 def _make_run_with_original(mod_name, func_name):
     @contextlib.contextmanager
     def with_orig():
@@ -591,6 +788,13 @@ if WIN:
     disabled_tests += [
         # Issue with Unix vs DOS newlines in the file vs from the server
         'test_ssl.ThreadedTests.test_socketserver',
+        # This sometimes hangs (only on appveyor)
+        'test_ssl.ThreadedTests.test_asyncore_server',
+        # On appveyor, this sometimes produces 'A non-blocking socket
+        # operation could not be completed immediately', followed by
+        # 'No connection could be made because the target machine
+        # actively refused it'
+        'test_socket.NonBlockingTCPTests.testAccept',
     ]
 
     # These are a problem on 3.5; on 3.6+ they wind up getting (accidentally) disabled.
@@ -611,8 +815,52 @@ if PYPY:
         # On some platforms, this returns "zlib_compression", but the test is looking for
         # "ZLIB"
         'test_ssl.ThreadedTests.test_compression',
+
+        # These are flaxy, apparently a race condition? Began with PyPy 2.7-7 and 3.6-7
+        'test_asyncore.TestAPI_UsePoll.test_handle_error',
+        'test_asyncore.TestAPI_UsePoll.test_handle_read',
     ]
 
+    if WIN:
+        disabled_tests += [
+            # Starting in 7.3.1 on Windows, this stopped raising ValueError; it appears to
+            # be a bug in PyPy.
+            'test_signal.WakeupFDTests.test_invalid_fd',
+            # Likewise for 7.3.1. See the comments for PY35
+            'test_socket.GeneralModuleTests.test_sock_ioctl',
+        ]
+
+    if PY36:
+        disabled_tests += [
+            # These are flaky, beginning in 3.6-alpha 7.0, not finding some flag
+            # set, apparently a race condition
+            'test_asyncore.TestAPI_UveIPv6Poll.test_handle_accept',
+            'test_asyncore.TestAPI_UveIPv6Poll.test_handle_accepted',
+            'test_asyncore.TestAPI_UveIPv6Poll.test_handle_close',
+            'test_asyncore.TestAPI_UveIPv6Poll.test_handle_write',
+
+            'test_asyncore.TestAPI_UseIPV6Select.test_handle_read',
+
+            # These are reporting 'ssl has no attribute ...'
+            # This could just be an OSX thing
+            'test_ssl.ContextTests.test__create_stdlib_context',
+            'test_ssl.ContextTests.test_create_default_context',
+            'test_ssl.ContextTests.test_get_ciphers',
+            'test_ssl.ContextTests.test_options',
+            'test_ssl.ContextTests.test_constants',
+
+            # These tend to hang for some reason, probably not properly
+            # closed sockets.
+            'test_socketserver.SocketServerTest.test_write',
+
+            # This uses ctypes to do funky things including using ptrace,
+            # it hangs
+            'test_subprocess.ProcessTestcase.test_child_terminated_in_stopped_state',
+
+            # Certificate errors; need updated test
+            'test_urllib2_localnet.TestUrlopen.test_https',
+        ]
+
 # Generic Python 3
 
 if PY3:
@@ -683,9 +931,13 @@ if PY3:
             'test_subprocess.ProcessTestCaseNoPoll.test_cwd_with_relative_arg',
             'test_subprocess.ProcessTestCase.test_cwd_with_relative_executable',
 
+            # In 3.7 and 3.8 on Travis CI, this appears to take the full 3 seconds.
+            # Can't reproduce it locally. We have our own copy of this that takes
+            # timing on CI into account.
+            'test_subprocess.RunFuncTestCase.test_run_with_shell_timeout_and_capture_output',
         ]
 
-    wrapped_tests.update({
+    disabled_tests += [
         # XXX: BUG: We simply don't handle this correctly. On CPython,
         # we wind up raising a BlockingIOError and then
         # BrokenPipeError and then some random TypeErrors, all on the
@@ -693,13 +945,20 @@ if PY3:
         # socket.makefile), whereas CPython 3.6 uses socket.sendall().
         # On PyPy, the behaviour is much worse: we hang indefinitely, perhaps exposing a problem
         # with our signal handling.
+
         # In actuality, though, this test doesn't fully test the EINTR it expects
         # to under gevent (because if its EWOULDBLOCK retry behaviour.)
         # Instead, the failures were all due to `pthread_kill` trying to send a signal
         # to a greenlet instead of a real thread. The solution is to deliver the signal
-        # to the real thread by letting it get the correct ID.
-        'test_wsgiref.IntegrationTests.test_interrupted_write': _make_run_with_original('threading', 'get_ident')
-    })
+        # to the real thread by letting it get the correct ID, and we previously
+        # used make_run_with_original to make it do that.
+        #
+        # But now that we have disabled our wrappers around Thread.join() in favor
+        # of the original implementation, that causes problems:
+        # background.join() thinks that it is the current thread, and won't let it
+        # be joined.
+        'test_wsgiref.IntegrationTests.test_interrupted_write',
+    ]
 
 # PyPy3 3.5.5 v5.8-beta
 
@@ -729,6 +988,9 @@ if PYPY and PY3:
         # (at least on OS X; it's less consistent about that on travis)
         'test_ssl.NetworkedBIOTests.test_handshake',
 
+        # This passes various "invalid" strings and expects a ValueError. not sure why
+        # we don't see errors on CPython.
+        'test_subprocess.ProcessTestCase.test_invalid_env',
     ]
 
     if OSX:
@@ -740,6 +1002,7 @@ if PYPY and PY3:
             'test_subprocess.POSIXProcessTestCase.test_pass_fds_inheritable',
             'test_subprocess.POSIXProcessTestCase.test_pipe_cloexec',
 
+
             # The below are new with 5.10.1
             # These fail with 'OSError: received malformed or improperly truncated ancillary data'
             'test_socket.RecvmsgSCMRightsStreamTest.testCmsgTruncLen0',
@@ -758,12 +1021,16 @@ if PYPY and PY3:
             'test_ssl.ThreadedTests.test_protocol_sslv3',
             'test_ssl.ThreadedTests.test_protocol_tlsv1',
             'test_ssl.ThreadedTests.test_protocol_tlsv1_1',
+            # Similar, they fail without monkey-patching.
+            'test_ssl.TestPostHandshakeAuth.test_pha_no_pha_client',
+            'test_ssl.TestPostHandshakeAuth.test_pha_optional',
+            'test_ssl.TestPostHandshakeAuth.test_pha_required',
 
             # This gets None instead of http1.1, even without gevent
             'test_ssl.ThreadedTests.test_npn_protocols',
 
             # This fails to decode a filename even without gevent,
-            # at least on High Sierarr.
+            # at least on High Sierra. Newer versions of the tests actually skip this.
             'test_httpservers.SimpleHTTPServerTestCase.test_undecodable_filename',
         ]
 
@@ -806,19 +1073,41 @@ if PYPY:
         # This is an important test, so rather than skip it in patched_tests_setup,
         # we do the gc before we return.
         'test_urllib2_localnet.TestUrlopen.test_https_with_cafile': _gc_at_end,
+
+        'test_httpservers.BaseHTTPServerTestCase.test_command': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_handler': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_head_keep_alive': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_head_via_send_error': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_header_close': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_internal_key_error': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_request_line_trimming': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_return_custom_status': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_return_header_keep_alive': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_send_blank': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_send_error': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_version_bogus': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_version_digits': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_version_invalid': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_version_none': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_version_none_get': _gc_at_end,
+        'test_httpservers.BaseHTTPServerTestCase.test_get': _gc_at_end,
+        'test_httpservers.SimpleHTTPServerTestCase.test_get': _gc_at_end,
+        'test_httpservers.SimpleHTTPServerTestCase.test_head': _gc_at_end,
+        'test_httpservers.SimpleHTTPServerTestCase.test_invalid_requests': _gc_at_end,
+        'test_httpservers.SimpleHTTPServerTestCase.test_path_without_leading_slash': _gc_at_end,
+        'test_httpservers.CGIHTTPServerTestCase.test_invaliduri': _gc_at_end,
+        'test_httpservers.CGIHTTPServerTestCase.test_issue19435': _gc_at_end,
+
+        'test_httplib.TunnelTests.test_connect': _gc_at_end,
+        'test_httplib.SourceAddressTest.testHTTPConnectionSourceAddress': _gc_at_end,
+
+        # Unclear
+        'test_urllib2_localnet.ProxyAuthTests.test_proxy_with_bad_password_raises_httperror': _gc_at_end,
+        'test_urllib2_localnet.ProxyAuthTests.test_proxy_with_no_password_raises_httperror': _gc_at_end,
     })
 
 
-if PY34 and sys.version_info[:3] < (3, 4, 4):
-    # Older versions have some issues with the SSL tests. Seen on Appveyor
-    disabled_tests += [
-        'test_ssl.ContextTests.test_options',
-        'test_ssl.ThreadedTests.test_protocol_sslv23',
-        'test_ssl.ThreadedTests.test_protocol_sslv3',
-        'test_httplib.HTTPSTest.test_networked',
-    ]
-
-if PY34:
+if PY35:
     disabled_tests += [
         'test_subprocess.ProcessTestCase.test_threadsafe_wait',
         # XXX: It seems that threading.Timer is not being greened properly, possibly
@@ -854,10 +1143,6 @@ if PY34:
         # it should be found at runtime.
         'test_socket.GeneralModuleTests.test_sock_ioctl',
 
-        # See comments for 2.7; these hang
-        'test_httplib.HTTPSTest.test_local_good_hostname',
-        'test_httplib.HTTPSTest.test_local_unknown_cert',
-
         # XXX This fails for an unknown reason
         'test_httplib.HeaderTests.test_parse_all_octets',
     ]
@@ -869,28 +1154,6 @@ if PY34:
             'test_socket.RecvmsgSCMRightsStreamTest.testFDPassEmpty',
         ]
 
-    if sys.version_info[:2] == (3, 4):
-        disabled_tests += [
-            # These are all expecting that a signal (sigalarm) that
-            # arrives during a blocking call should raise
-            # InterruptedError with errno=EINTR. gevent does not do
-            # this, instead its loop keeps going and raises a timeout
-            # (which fails the test). HOWEVER: Python 3.5 fixed this
-            # problem and started raising a timeout,
-            # (https://docs.python.org/3/whatsnew/3.5.html#pep-475-retry-system-calls-failing-with-eintr)
-            # and removed these tests (InterruptedError is no longer
-            # raised). So basically, gevent was ahead of its time.
-            'test_socket.InterruptedRecvTimeoutTest.testInterruptedRecvIntoTimeout',
-            'test_socket.InterruptedRecvTimeoutTest.testInterruptedRecvTimeout',
-            'test_socket.InterruptedRecvTimeoutTest.testInterruptedRecvfromIntoTimeout',
-            'test_socket.InterruptedRecvTimeoutTest.testInterruptedRecvfromTimeout',
-            'test_socket.InterruptedRecvTimeoutTest.testInterruptedSendTimeout',
-            'test_socket.InterruptedRecvTimeoutTest.testInterruptedSendtoTimeout',
-            'test_socket.InterruptedRecvTimeoutTest.testInterruptedRecvmsgTimeout',
-            'test_socket.InterruptedRecvTimeoutTest.testInterruptedRecvmsgIntoTimeout',
-            'test_socket.InterruptedSendTimeoutTest.testInterruptedSendmsgTimeout',
-        ]
-
         if TRAVIS:
             # This has been seen to produce "Inconsistency detected by
             # ld.so: dl-open.c: 231: dl_open_worker: Assertion
@@ -898,6 +1161,9 @@ if PY34:
             # RT_CONSISTENT' failed!" and fail.
             disabled_tests += [
                 'test_threading.ThreadTests.test_is_alive_after_fork',
+                # This has timing constraints that are strict and do not always
+                # hold.
+                'test_selectors.PollSelectorTestCase.test_timeout',
             ]
 
     if TRAVIS:
@@ -923,9 +1189,6 @@ if PY35:
         # 'lock_tests.LockTests.lest_locked_repr',
         # 'lock_tests.LockTests.lest_repr',
 
-        # Added between 3.6.0 and 3.6.3, uses _testcapi and internals
-        # of the subprocess module.
-        'test_subprocess.POSIXProcessTestCase.test_stopped',
 
         # This test opens a socket, creates a new socket with the same fileno,
         # closes the original socket (and hence fileno) and then
@@ -937,6 +1200,14 @@ if PY35:
         'test_socket.TestExceptions.test_setblocking_invalidfd',
     ]
 
+    if sys.version_info[:2] == (3, 5):
+        # These tests are broken now that certificates are
+        # expired and Python 3.5 is out of maintenance.
+        disabled_tests += [
+            'test_ssl.ThreadedTests.test_crl_check',
+            'test_ssl.BasicSocketTests.test_parse_cert',
+        ]
+
     if ARES:
         disabled_tests += [
             # These raise different errors or can't resolve
@@ -1023,13 +1294,85 @@ if PY37:
         # but it passes when they run it and fails when we do. It's not
         # clear why.
         'test_ssl.ThreadedTests.test_check_hostname_idn',
+
+        # These appear to hang, haven't investigated why
+        'test_ssl.SimpleBackgroundTests.test_get_server_certificate',
+        # Probably the same as NetworkConnectionNoServer.test_create_connection_timeout
+        'test_socket.NetworkConnectionNoServer.test_create_connection',
+
+        # Internals of the threading module that change.
+        'test_threading.ThreadTests.test_finalization_shutdown',
+        'test_threading.ThreadTests.test_shutdown_locks',
+        # Expects a deprecation warning we don't raise
+        'test_threading.ThreadTests.test_old_threading_api',
+        # This tries to use threading.interrupt_main() from a new Thread;
+        # but of course that's actually the same thread and things don't
+        # work as expected.
+        'test_threading.InterruptMainTests.test_interrupt_main_subthread',
+        'test_threading.InterruptMainTests.test_interrupt_main_noerror',
+
+        # TLS1.3 seems flaky
+        'test_ssl.ThreadedTests.test_wrong_cert_tls13',
     ]
 
+    if sys.version_info < (3, 7, 6):
+        disabled_tests += [
+            # Earlier versions parse differently so the newer test breaks
+            'test_ssl.BasicSocketTests.test_parse_all_sans',
+            'test_ssl.BasicSocketTests.test_parse_cert_CVE_2013_4238',
+        ]
+
     if APPVEYOR:
         disabled_tests += [
-
+            # This sometimes produces ``self.assertEqual(1, len(s.select(0))): 1 != 0``.
+            # Probably needs to spin the loop once.
+            'test_selectors.BaseSelectorTestCase.test_timeout',
         ]
 
+if PY38:
+    disabled_tests += [
+        # This one seems very strict: doesn't want a pathlike
+        # first argument when shell is true.
+        'test_subprocess.RunFuncTestCase.test_run_with_pathlike_path',
+        # This tests for a warning we don't raise.
+        'test_subprocess.RunFuncTestCase.test_bufsize_equal_one_binary_mode',
+
+        # This compares the output of threading.excepthook with
+        # data constructed in Python. But excepthook is implemented in C
+        # and can't see the patched threading.get_ident() we use, so the
+        # output doesn't match.
+        'test_threading.ExceptHookTests.test_excepthook_thread_None',
+    ]
+
+    if sys.version_info[:3] < (3, 8, 1):
+        disabled_tests += [
+            # Earlier versions parse differently so the newer test breaks
+            'test_ssl.BasicSocketTests.test_parse_all_sans',
+            'test_ssl.BasicSocketTests.test_parse_cert_CVE_2013_4238',
+        ]
+
+    if sys.version_info[:3] < (3, 8, 10):
+        disabled_tests += [
+            # These were added for fixes sometime between 3.8.1 and 3.8.10
+            'test_ftplib.TestFTPClass.test_makepasv_issue43285_security_disabled',
+            'test_ftplib.TestFTPClass.test_makepasv_issue43285_security_enabled_default',
+            'test_httplib.BasicTest.test_dir_with_added_behavior_on_status',
+            'test_httplib.TunnelTests.test_tunnel_connect_single_send_connection_setup',
+            'test_ssl.TestSSLDebug.test_msg_callback_deadlock_bpo43577',
+            # This one fails with the updated certs
+            'test_ssl.ContextTests.test_load_verify_cadata',
+            # This one times out on 3.7.1 on Appveyor
+            'test_ftplib.TestTLS_FTPClassMixin.test_retrbinary_rest',
+        ]
+
+if RESOLVER_DNSPYTHON:
+    disabled_tests += [
+        # This does two things DNS python doesn't. First, it sends it
+        # capital letters and expects them to be returned lowercase.
+        # Second, it expects the symbolic scopeid to be stripped from the end.
+        'test_socket.GeneralModuleTests.test_getaddrinfo_ipv6_scopeid_symbolic',
+    ]
+
 # if 'signalfd' in os.environ.get('GEVENT_BACKEND', ''):
 #     # tests that don't interact well with signalfd
 #     disabled_tests.extend([
@@ -1047,6 +1390,110 @@ disabled_tests += [
     'test_ssl.BasicSocketTests.test_openssl_version'
 ]
 
+if OSX:
+
+    disabled_tests += [
+        # This sometimes produces OSError: Errno 40: Message too long
+        'test_socket.RecvmsgIntoTCPTest.testRecvmsgIntoGenerator',
+
+        # These sometime timeout. Cannot reproduce locally.
+        'test_ftp.TestTLS_FTPClassMixin.test_mlsd',
+        'test_ftp.TestTLS_FTPClassMixin.test_retrlines_too_long',
+        'test_ftp.TestTLS_FTPClassMixin.test_storlines',
+        'test_ftp.TestTLS_FTPClassMixin.test_retrbinary_rest',
+    ]
+
+    if RESOLVER_ARES and PY38 and not RUNNING_ON_CI:
+        disabled_tests += [
+            # When updating to 1.16.0 this was seen locally, but not on CI.
+            # Tuples differ: ('ff02::1de:c0:face:8d', 1234, 0, 0)
+            #             != ('ff02::1de:c0:face:8d', 1234, 0, 1)
+            'test_socket.GeneralModuleTests.test_getaddrinfo_ipv6_scopeid_symbolic',
+        ]
+
+if PY39:
+
+    disabled_tests += [
+        # Depends on exact details of the repr. Eww.
+        'test_subprocess.ProcessTestCase.test_repr',
+        # Tries to wait for the process without using Popen APIs, and expects the
+        # ``returncode`` attribute to stay None. But we have already hooked SIGCHLD, so
+        # we see and set the ``returncode``; there is no way to wait that doesn't do that.
+        'test_subprocess.POSIXProcessTestTest.test_send_signal_race',
+    ]
+
+    if sys.version_info[:3] < (3, 9, 5):
+        disabled_tests += [
+            # These were added for fixes sometime between 3.9.1 and 3.9.5
+            'test_ftplib.TestFTPClass.test_makepasv_issue43285_security_disabled',
+            'test_ftplib.TestFTPClass.test_makepasv_issue43285_security_enabled_default',
+            'test_httplib.BasicTest.test_dir_with_added_behavior_on_status',
+            'test_httplib.TunnelTests.test_tunnel_connect_single_send_connection_setup',
+            'test_ssl.TestSSLDebug.test_msg_callback_deadlock_bpo43577',
+            # This one fails with the updated certs
+            'test_ssl.ContextTests.test_load_verify_cadata',
+            # These time out on 3.9.1 on Appveyor
+            'test_ftplib.TestTLS_FTPClassMixin.test_retrbinary_rest',
+            'test_ftplib.TestTLS_FTPClassMixin.test_retrlines_too_long',
+        ]
+
+if PY310:
+    disabled_tests += [
+        # They arbitrarily made some types so that they can't be created;
+        # that's an implementation detail we're not going to follow (
+        # it would require them to be factory functions).
+        'test_select.SelectTestCase.test_disallow_instantiation',
+        'test_threading.ThreadTests.test_disallow_instantiation',
+        # This wants two true threads to work, but a CPU bound loop
+        # in a greenlet can't be interrupted.
+        'test_threading.InterruptMainTests.test_can_interrupt_tight_loops',
+        # We don't currently implement pipesize.
+        'test_subprocess.ProcessTestCase.test_pipesize_default',
+        'test_subprocess.ProcessTestCase.test_pipesizes',
+        # Unknown
+        'test_signal.SiginterruptTest.test_siginterrupt_off',
+    ]
+
+    if TRAVIS:
+        disabled_tests += [
+            # The mixing of subinterpreters (with threads) and gevent apparently
+            # leads to a segfault on Ubuntu/GitHubActions/3.10rc1. Not clear why.
+            # But that's not a great use case for gevent.
+            'test_threading.SubinterpThreadingTests.test_threads_join',
+            'test_threading.SubinterpThreadingTests.test_threads_join_2',
+        ]
+
+if PY311:
+    disabled_tests += [
+        # CPython issue #27718: This wants to require all objects to
+        # have a __module__ of 'signal' because pydoc. Obviously our patches don't.
+        'test_signal.GenericTests.test_functions_module_attr',
+        # 3.11 added subprocess._USE_VFORK and subprocess._USE_POSIX_SPAWN.
+        # We don't support either of those (although USE_VFORK might be possible?)
+        'test_subprocess.ProcessTestCase.test__use_vfork',
+    ]
+
+if TRAVIS:
+    disabled_tests += [
+        # These tests frequently break when we try to use newer Travis CI images,
+        # due to different versions of OpenSSL being available. See above for some
+        # specific examples. Usually the tests catch up, eventually (e.g., at this writing,
+        # the 3.9b1 tests are fine on Ubuntu Bionic, but all other versions fail).
+        'test_ssl.ContextTests.test_options',
+        'test_ssl.ThreadedTests.test_alpn_protocols',
+        'test_ssl.ThreadedTests.test_default_ecdh_curve',
+        'test_ssl.ThreadedTests.test_shared_ciphers',
+
+    ]
+
+if RUNNING_ON_MUSLLINUX:
+    disabled_tests += [
+        # This is supposed to *not* crash, but on the muslilnux image, it
+        # does crash.
+        'test_threading.ThreadingExceptionTests.test_recursion_limit',
+    ]
+
+
 # Now build up the data structure we'll use to actually find disabled tests
 # to avoid a linear scan for every file (it seems the list could get quite large)
 # (First, freeze the source list to make sure it isn't modified anywhere)
